# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class ObjectUser(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_object_users(self, namespace=None):
        """
        Gets identifiers for all configured users. If namespace is provided
        then returns all users for the specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'blobuser': [
                {
                    u'userid': u'johndoe',
                    u'namespace': u'namespace1'
                },
                {
                    u'userid': u'janedoe',
                    u'namespace': u'namespace1'
                }
            ]
        }

        :param namespace: Example: namespace1
        """
        msg = 'Getting all users'
        url = 'object/users'

        if namespace:
            url += '/{0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def get_object_user_info(self, uid, namespace=None):
        """
        Gets user details for the specified user.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'locked': False,
            u'namespace': u'namespace1',
            u'name': u'someone',
            u'created': u'ThuMay2105: 43: 27UTC2015'
        }

        :param uid: Valid user identifier
        :param namespace: Optional when userscope is GLOBAL. Required when
        userscope is NAMESPACE. The namespace to which the user belongs
        """
        msg = "Getting user '{0}'".format(uid)
        url = 'object/users/{0}/info'.format(uid)

        if namespace:
            url += '?namespace={0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def deactivate_object_user(self, uid, namespace=None):
        """
        Deletes the specified user and its secret keys.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param uid: Valid user identifier
        :param namespace: Example: namespace1 (optional)
        """
        if namespace:
            payload = {
                "user": uid,
                "namespace": namespace
            }
        else:
            payload = {
                "user": uid
            }

        log.info("Deleting user: {0}".format(payload))

        return self.conn.post(url='object/users/deactivate',
                              json_payload=payload)

    def add_object_user(self, uid, namespace, tags=None):
        """
        Creates a user for a specified namespace. The user must subsequently
        be assigned a secret key in order to access the object store.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'isLocked': True,
            u'user_name': u'janedoe'
        }

        :param uid: Valid user identifier
        :param namespace: Example: namespace1
        """

        payload = {
            "user": uid,
            "namespace": namespace
        }

        if tags:
            payload['tags'] = tags

        log.info('Creating user: {0}'.format(payload))
        log.info("(don't forget to create secret key)")

        return self.conn.post(url='object/users',
                              json_payload=payload)

    def lock_object_user(self, uid, is_locked=True, namespace=None):
        """
        Locks or unlocks the specified user. If the user belongs to a
        namespace, the namespace must be supplied.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param uid: Valid user identifier
        :param is_locked: Example: True/False (optional)
        :param namespace: Example: namespace1 (optional)
        """
        if namespace:
            payload = {
                "user": uid,
                "namespace": namespace,
                "isLocked": is_locked
            }
        else:
            payload = {
                "user": uid,
                "isLocked": is_locked
            }

        log.info("Toggling user lock: {0}".format(payload))

        return self.conn.put(url='object/users/lock',
                             json_payload=payload)

    def get_object_user_lock(self, uid, namespace=None):
        """
        Gets the user lock state for the specified user belonging to the
        specified namespace (if provided).

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'user_name': u'testlogin',
            u'isLocked': False
        }

        :param uid: Valid user identifier
        :param namespace: Example: namespace1
        """
        msg = "Getting lock state for user '{0}'".format(uid)
        url = 'object/users/lock/{0}'.format(uid)

        if namespace:
            url += '/{0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class ReplicationGroup(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_replication_groups(self):
        """
        Lists all configured replication groups.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'data_service_vpool': [
                {
                    u'isAllowAllNamespaces': True,
                    u'remote': None,
                    u'name': u'us1',
                    u'tags': [

                    ],
                    u'global': None,
                    u'creation_time': 1434739694878,
                    u'vdc': None,
                    u'inactive': False,
                    u'varrayMappings': [
                        {
                            u'name': u'urn: storageos: VirtualDataCenterData:
                                    7d7d2ed5-e253-4be9-b8bb-5ff5b2697e6f',
                            u'value': u'urn: storageos: VirtualArray:
                                    527caf55-e989-4485-b92d-ce465d20dd56'
                        }
                    ],
                    u'id': u'urn: storageos: ReplicationGroupInfo:
                                61f20dc2-a862-4935-9110-45030f0fe17c: global',
                    u'description': u''
                },
                {
                    u'isAllowAllNamespaces': True,
                    u'remote': None,
                    u'name': u'us2',
                    u'tags': [

                    ],
                    u'global': None,
                    u'creation_time': 1434739694878,
                    u'vdc': None,
                    u'inactive': False,
                    u'varrayMappings': [
                        {
                            u'name': u'urn: storageos: VirtualDataCenterData:
                                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
                            u'value': u'urn: storageos: VirtualArray:
                                        3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37'
                        }
                    ],
                    u'id': u'urn: storageos: ReplicationGroupInfo:
                            c2b0d3c4-c778-4a24-8da5-6a89784c4eeb: global',
                    u'description': u''
                }
            ]
        }
        """
        log.info("Fetching list of replication groups")
        return self.conn.get(url='vdc/data-service/vpools')

    def get_replication_group(self, replication_group_id):
        """
        Gets the details for the specified replication group.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'isAllowAllNamespaces': True,
            u'remote': None,
            u'name': u'us2',
            u'tags': [

            ],
            u'global': None,
            u'creation_time': 1434740060384,
            u'vdc': None,
            u'inactive': False,
            u'varrayMappings': [
                {
                    u'name': u'urn: storageos: VirtualDataCenterData:
                                a9faea85-d377-4a42-b5f1-fa15829f0c33',
                    u'value': u'urn: storageos: VirtualArray:
                                3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37'
                }
            ],
            u'id': u'urn: storageos: ReplicationGroupInfo:
                        c2b0d3c4-c778-4a24-8da5-6a89784c4eeb: global',
            u'description': u''
        }

        :param replication_group_id: Replication group identifier for which
        details needs to be retrieved
        """
        log.info("Fetching vpool ID '{0}'".format(replication_group_id))

        return self.conn.get(
            url='vdc/data-service/vpools/{0}'.format(replication_group_id))

    def update_replication_group(self, replication_group_id, name, description,
                                 allow_all_namespaces=False):
        """
        Updates the name and description for a replication group.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param replication_group_id: Replication group identifier to update
        :param name: New name for the replication group
        :param description: New description for the replication group
        :param allow_all_namespaces: Allow all namespaces to update True/False
        """
        payload = {
            "name": name,
            "description": description,
            "allow_all_namespaces": allow_all_namespaces
        }

        log.info("Updating vpool '{0}' with: {1}".format(replication_group_id,
                                                         payload))
        return self.conn.put(
            url='vdc/data-service/vpools/{0}'.format(replication_group_id),
            json_payload=payload
        )



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class ECSMinionException(Exception):
    """
    This is a custom ECSMinionException Exception class to better handle
    errors that ECSMinionException can return in HTML format rather than JSON
    """

    def __init__(self, message=None, http_status_code=None, ecs_message=None):
        """
        This is a custom ECSMinion Exception class to handle

        :param message: A custom
        :param http_status_code:
        :param ecs_message:
        :return:
        """
        if message is None:
            self.message = 'The ECSMinionException endpoint has thrown ' \
                           'an error, check the http_status_code and ' \
                           'ecs_message attributes of this exception for ' \
                           'more details.'
        else:
            self.message = message

        self.http_status_code = http_status_code
        self.ecs_message = ecs_message

        super(ECSMinionException, self).__init__(
            message, http_status_code, ecs_message)



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class SecretKey(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_user_secret_keys(self, uid, namespace=None):
        """
        Gets all secret keys for the specified user.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'key_timestamp_2': u'2015-05-2105: 43: 33.281',
            u'key_timestamp_1': u'2015-05-2105: 43: 30.872',
            u'key_expiry_timestamp_2': u'',
            u'key_expiry_timestamp_1': u'',
            u'secret_key_1': u'3tW4A5G7QaAGVtsGLVL8uDy73S2A6LrvkPTDNdrk',
            u'secret_key_2': u'v5LjgoEM9lqHIo+qa7pIhGjrecc/Wv+WVw7kO4oQ',
            u'link': {
                u'href': u'/object/secret-keys',
                u'rel': u'self'
            }
        }

        :param uid: Valid user identifier to get the keys from
        :param namespace: The namespace
        """
        msg = "getting secret keys for user '{0}'".format(uid)
        url = 'object/user-secret-keys/{0}'.format(uid)

        if namespace:
            url += '/{0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def create_new_secret_key(self, uid, namespace=None,
                              key_expiration=2592000, secret_key=None):
        """
        Creates a secret key for the specified user. If the user belongs to a
        namespace, the namespace must be supplied. When creating a new secret
        key, you may pass in an expiration time in minutes for the old key.
        During the expiration interval, both keys will be accepted for
        requests. This gives you a grace period where you can update
        applications to use the new key.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            "link": {
                "rel": "self",
                "href":"/object/user-secret-keys/joedeleteme"
            },
            "secret_key": "p3PZyb//Ch6tM0fUsnesYYnGb+6JHV8WHzS5YHjg",
            "key_timestamp": "2014-12-24 02:08:40.181"
        }

        :param uid: Valid user identifier to create a key for
        :param namespace: The namespace
        :param key_expiration: Defaults to 30 days (2592000 seconds)
        :param secret_key: Manually specify the new secret key
        """
        payload = {
            "existing_key_expiry_time_mins": key_expiration,
            "namespace": namespace
        }

        if secret_key:
            payload['secretkey'] = secret_key

        log.info("Creating secret for user '{0}': {1}".format(uid, payload))

        return self.conn.post(url='object/user-secret-keys/{0}'.format(uid),
                              json_payload=payload)

    def deactivate_user_secret_key(self, uid, namespace=None,
                                   secret_key=None):
        """
        Deletes all secret keys for the specific user. If the user belongs
        namespace, the namespace must be supplied.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param uid: Valid user identifier to get delete the keys from
        :param namespace: The namespace
        :param secret_key: The secret key to deactivate
        """
        payload = {
            "secret_key": secret_key,
            "namespace": namespace
        }

        log.info("Deleting secret for user '{0}': {1}".format(uid, payload))

        return self.conn.post(
            url='object/user-secret-keys/{0}/deactivate'.format(uid),
            json_payload=payload)



# Standard lib imports
import json
import logging
import os

# Third party imports
import requests

# Project level imports
from ecsminion.api.authentication import Authentication

from ecsminion.api.configuration.certificate import Certificate
from ecsminion.api.configuration.configuration_properties \
    import ConfigurationProperties
from ecsminion.api.configuration.licensing import Licensing

from ecsminion.api.metering.billing import Billing

from ecsminion.api.monitoring.capacity import Capacity
from ecsminion.api.monitoring.dashboard import Dashboard
from ecsminion.api.monitoring.events import Events

from ecsminion.api.multitenancy.namespace import Namespace

from ecsminion.api.provisioning.base_url import BaseUrl
from ecsminion.api.provisioning.bucket import Bucket
from ecsminion.api.provisioning.data_store import DataStore
from ecsminion.api.provisioning.node import Node
from ecsminion.api.provisioning.storage_pool import StoragePool
from ecsminion.api.provisioning.virtual_data_center import VirtualDataCenter

from ecsminion.api.support.call_home import CallHome

from ecsminion.api.geo_replication.replication_group import ReplicationGroup
from ecsminion.api.geo_replication.temporary_failed_zone import \
    TemporaryFailedZone

from ecsminion.api.user_management.authentication_provider import \
    AuthenticationProvider
from ecsminion.api.user_management.secret_key import SecretKey
from ecsminion.api.user_management.secret_key_self_service import \
    SecretKeySelfService
from ecsminion.api.user_management.user_management import ManagementUser
from ecsminion.api.user_management.user_object import ObjectUser

from ecsminion.api.undocumented.user_info import UserInfo

from ecsminion.util.exceptions import ECSMinionException
from ecsminion.util.token_request import TokenRequest


# Suppress the insecure request warning
# https://urllib3.readthedocs.org/en/
# latest/security.html#insecurerequestwarning
requests.packages.urllib3.disable_warnings()

# Initialize logger
log = logging.getLogger(__name__)


class ECSMinion(object):

    def __init__(self, username=None, password=None, token=None,
                 ecs_endpoint=None, token_endpoint=None, verify_ssl=False,
                 token_filename='ecsminion.tkn', token_location='/tmp',
                 request_timeout=15.0, cache_token=True):
        """
        Creates the ECSMinion class that the client will directly work with

        :param username: The username to fetch a token
        :param password: The password to fetch a token
        :param token: Supply a valid token to use instead of username/password
        :param ecs_endpoint: The URL where ECS is located
        :param token_endpoint: The URL where the ECS login is located
        :param verify_ssl: Verify SSL certificates
        :param token_filename: The name of the cached token filename
        :param token_location: By default this is stored in /tmp
        :param request_timeout: How long to wait for ECS to respond
        :param cache_token: Whether to cache the token, by default this is true
        you should only switch this to false when you want to directly fetch
        a token for a user
        """

        self.username = username
        self.password = password
        self.token = token
        self.ecs_endpoint = ecs_endpoint.rstrip('/')
        self.token_endpoint = token_endpoint.rstrip('/')
        self.verify_ssl = verify_ssl
        self.token_filename = token_filename
        self.token_location = token_location
        self.request_timeout = request_timeout
        self.cache_token = cache_token
        self._session = requests.Session()
        self._token_request = TokenRequest(
            username=self.username,
            password=self.password,
            ecs_endpoint=self.ecs_endpoint,
            token_endpoint=self.token_endpoint,
            verify_ssl=self.verify_ssl,
            token_filename=self.token_filename,
            token_location=self.token_location,
            request_timeout=self.request_timeout,
            cache_token=self.cache_token)
        self.token_file = os.path.join(
            self.token_location, self.token_filename)

        # API -> Authentication
        self.authentication = Authentication(self)

        # API -> Billing
        self.billing = Billing(self)

        # API -> Configuration
        self.certificate = Certificate(self)
        self.configuration_properties = ConfigurationProperties(self)
        self.licensing = Licensing(self)

        # API -> Geo Replication
        self.replication_group = ReplicationGroup(self)
        self.temp_failed_zone = TemporaryFailedZone(self)

        # API -> Monitoring
        self.capacity = Capacity(self)
        self.dashboard = Dashboard(self)
        self.events = Events(self)

        # API -> Monitoring
        self.namespace = Namespace(self)

        # API -> Provisioning
        self.base_url = BaseUrl(self)
        self.bucket = Bucket(self)
        self.data_store = DataStore(self)
        self.node = Node(self)
        self.storage_pool = StoragePool(self)
        self.virtual_data_center = VirtualDataCenter(self)

        # API -> Support
        self.call_home = CallHome(self)

        # API -> User Management
        self.authentication_provider = AuthenticationProvider(self)
        self.secret_key = SecretKey(self)
        self.secret_key_self_service = SecretKeySelfService(self)
        self.management_object = ManagementUser(self)
        self.user_object = ObjectUser(self)

        # API -> Undocumented
        self.user_info = UserInfo(self)

    def get_token(self):
        """
        Get a token directly back, typically you want to set the cache_token
        param for ecsminion to false for this call.

        :return: A valid token or an ecsminion exception
        """
        return self._token_request.get_new_token()

    def remove_cached_token(self):
        """
        Remove the cached token file, this is useful if you switch users
        and want to use a different token
        """
        if os.path.isfile(self.token_file):
            log.debug("Removing cached token '{0}'".format(self.token_file))
            os.remove(self.token_file)

    def _fetch_headers(self):
        token = self.token if self.token else self._token_request.get_token()
        return {'Accept': 'application/json',
                'Content-Type': 'application/json',
                'x-sds-auth-token': token}

    def _construct_url(self, path):
        url = '{0}/{1}'.format(self.ecs_endpoint, path)
        log.debug('Constructed URL as: {0}'.format(url))
        return url

    def get(self, url, params=None):
        return self._request(url, params=params)

    def post(self, url, json_payload='{}'):
        return self._request(url, json_payload, http_verb='POST')

    def put(self, url, json_payload='{}'):
        return self._request(url, json_payload, http_verb='PUT')

    def delete(self, url, params=None):
        return self._request(url, params=params, http_verb='DELETE')

    def _request(self, url, json_payload='{}', http_verb='GET', params=None):
        json_payload = json.dumps(json_payload)

        try:
            if http_verb == "PUT":
                req = self._session.put(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    data=json_payload)
            elif http_verb == 'POST':
                req = self._session.post(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    data=json_payload)
            elif http_verb == 'DELETE':
                # Need to follow up - if 'accept' is in the headers
                # delete calls are not working because ECS 2.0 is returning
                # XML even is JSON is specified
                headers = self._fetch_headers()
                del headers['Accept']

                req = self._session.delete(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=headers,
                    timeout=self.request_timeout,
                    params=params)
            else:  # Default to GET
                req = self._session.get(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    params=params)

            if req.status_code != 200:
                log.error("Status code NOT OK")
                raise ECSMinionException(
                    http_status_code=req.status_code,
                    ecs_message=req.text)
            return req.json()

        except requests.ConnectionError as conn_err:
            msg = 'Connection error: {0}'.format(conn_err.args)
            log.error(msg)
            raise ECSMinionException(message=msg)
        except requests.HTTPError as http_err:
            msg = 'HTTP error: {0}'.format(http_err.args)
            log.error(msg)
            raise ECSMinionException(message=msg)
        except requests.RequestException as req_err:
            msg = 'Request error: {0}'.format(req_err.args)
            log.error(msg)
            raise ECSMinionException(message=msg)
        except ValueError:
            return



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class Bucket(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def create_bucket(self, bucket_name, vpool='', filesystem_enabled=False,
                      head_type=None, namespace=None, is_stale_allowed=False):
        """
        Creates a bucket which could be used by users to create objects.
        The bucket is created in a storage pool associated with the specified
        replication group.

        - Current user will become the bucket owner.
        - If namespace to this bucket creation does not exist, user's namespace
          is used
        - For non SYSTEM_ADMIN user, Namespace should be current user's
          namespace


        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'bucket-test1',
            u'tags': [],
            u'global': None,
            u'vdc': None,
            u'inactive': False,
            u'id': u'namespace1.bucket-test1'
        }

        :param bucket_name: The bucket name
        :param vpool:
        :param filesystem_enabled:
        :param head_type:
        :param namespace: The namespace
        :param is_stale_allowed:
        """
        payload = {
            "name": bucket_name,
            "vpool": vpool,
            "filesystem_enabled": filesystem_enabled,
            "head_type": head_type,
            "namespace": namespace,
            "is_stale_allowed": is_stale_allowed
        }

        log.info("Creating bucket '{0}': {1}".format(bucket_name, payload))

        if head_type:
            payload['head_type'] = head_type

        return self.conn.post(url='object/bucket', json_payload=payload)

    def deactivate_bucket(self, bucket_name, namespace=None):
        """
        Deletes the specified bucket.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: The bucket name
        :param namespace: The namespace
        """
        msg = "Deleting bucket '{0}'".format(bucket_name)
        url = 'object/bucket/{0}/deactivate'.format(bucket_name)

        if namespace:
            url += '?namespace={0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.post(url=url)

    def get_buckets(self, namespace, marker='', limit=100):
        """
        Gets the list of buckets for the specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'MaxBuckets': 100,
            u'NextMarker': u'blah-06860733-a0a8-45b5-b4ac-b18ee1e4257d',
            u'object_bucket': [
                {
                    u'remote': None,
                    u'name': u'my-test',
                    u'tags': [
                    ],
                    u'fs_access_enabled': False,
                    u'global': None,
                    u'namespace': u'namespace1',
                    u'api_type': u'S3',
                    u'created': u'2015-06-15T18: 39: 11.957Z',
                    u'vdc': None,
                    u'default_retention': -2,
                    u'is_stale_allowed': False,
                    u'owner': u'user1',
                    u'locked': False,
                    u'notification_size': -1,
                    u'block_size': -1,
                    u'vpool': u'urn: storageos: ReplicationGroupInfo:
                                3ba1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global'
                },
                {
                    u'remote': None,
                    u'name': u'large-uploads',
                    u'tags': [
                    ],
                    u'fs_access_enabled': False,
                    u'global': None,
                    u'namespace': u'namespace1',
                    u'api_type': u'S3',
                    u'created': u'2015-06-12T17: 35: 53.366Z',
                    u'vdc': None,
                    u'default_retention': -2,
                    u'is_stale_allowed': False,
                    u'owner': u'fancyuser1',
                    u'locked': False,
                    u'notification_size': -1,
                    u'block_size': -1,
                    u'vpool': u'urn: storageos: ReplicationGroupInfo:
                                3ba1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global'
                }
            ]
        }

        List of buckets associated with the namespace

        :param namespace: The namespace to query for buckets
        :param marker: Reference to last object returned
        :param limit: Number of objects requested in current fetch
        """
        log.info("Getting all buckets in namespace '{0}'".format(namespace))

        return self.conn.get(
            url='object/bucket?namespace={0}&marker={1}&limit={2}'.format(
                namespace, marker, limit))

    def set_bucket_retention(self, bucket_name, namespace, period=2592000):
        """
        Updates the default retention setting for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: The bucket name
        :param namespace: The namespace
        :param period: Default retention period for bucket in seconds. Defaults
        to 30 days (2592000 seconds)
        """
        payload = {
            "period": period,
            "namespace": namespace
        }

        log.info("Setting retention for bucket '{0}' in namespace "
                 "'{1}': {2}".format(bucket_name, namespace, payload))

        return self.conn.put(
            url='object/bucket/{0}/retention'.format(bucket_name),
            json_payload=payload
        )

    def get_bucket_retention(self, bucket_name, namespace=None):
        """
        Gets the retention setting for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {u'period': 2592000}

        :param bucket_name: Bucket name for which the retention setting will be
        retrieved
        :param namespace: Namespace associated. If it is null, then current
        user's Namespace is used.
        """
        msg = "Getting retention for bucket '{0}'".format(bucket_name)
        url = 'object/bucket/{0}/retention'.format(bucket_name)

        if namespace:
            url += '?namespace={0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def get_bucket_info(self, bucket_name, namespace=None):
        """
        Gets bucket information for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'bucket-test1',
            u'tags': [
            ],
            u'fs_access_enabled': False,
            u'global': None,
            u'namespace': u'namespace1',
            u'api_type': u'S3',
            u'created': u'2015-06-15T21: 26: 52.781Z',
            u'vdc': None,
            u'default_retention': 2592000,
            u'is_stale_allowed': False,
            u'owner': u'admin@internal',
            u'locked': False,
            u'notification_size': -1,
            u'block_size': -1,
            u'vpool': u'urn: storageos: ReplicationGroupInfo:
                        3af1fa7e-a7d1-4a9e-b8cc-e5a2c27f308d: global'
        }

        :param bucket_name: Bucket name for which information will be retrieved
        :param namespace: Namespace associated. If it is null, then current
        user's Namespace is used.
        """
        msg = "Getting info for bucket '{0}'".format(bucket_name)
        url = 'object/bucket/{0}/info'.format(bucket_name)

        if namespace:
            url += '?namespace={0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def update_bucket_owner(self, bucket_name, new_owner, namespace=None):
        """
        Updates the owner for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param namespace: The namespace
        :param new_owner: New owner must be a valid user
        :param bucket_name: Name of the bucket for which owner will be updated
        """
        payload = {
            "new_owner": new_owner
        }

        if namespace:
            payload['namespace'] = namespace

        log.info("Updating owner for bucket '{0}': {1}".format(bucket_name,
                                                               payload))
        return self.conn.post(
            url='object/bucket/{0}/owner'.format(bucket_name),
            json_payload=payload
        )

    def update_bucket_is_stale_allowed(self, bucket_name, is_stale_allowed,
                                       namespace=None):
        """
        Updates isStaleAllowed details for the specified bucket. If namespace
        does not exist in the request payload, the current user's namespace
        is used.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param namespace: The namespace
        :param is_stale_allowed: true/false
        :param bucket_name: Name of the bucket for which isStaleAllowed is to
        be updated
        """
        payload = {
            "is_stale_allowed": is_stale_allowed
        }

        if namespace:
            payload['namespace'] = namespace

        log.info("Updating 'isStaleAllowed' for bucket '{0}': {1}".format(
                  bucket_name, payload))

        return self.conn.post(
            url='object/bucket/{0}/isstaleallowed'.format(bucket_name),
            json_payload=payload
        )

    def get_bucket_lock(self, bucket_name, namespace=None):
        """
        Gets lock information for the specified bucket. The current user's
        namespace is used.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'isLocked': False,
            u'bucket_name': u'bucket-test1'
        }

        :param bucket_name: Example: The bucket name to get lock information
        :param namespace: Name of the bucket for which lock information is to
        be retrieved
        """
        msg = "Getting lock info for bucket '{0}'".format(bucket_name)
        url = 'object/bucket/{0}/lock'.format(bucket_name)

        if namespace:
            url += '?namespace={0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def set_lock_bucket(self, bucket_name, is_locked='false', namespace=None):
        """
        Locks or unlocks the specified bucket. Current user's namespace
        is used.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: Name of the bucket which is to be locked/unlocked.
        :param is_locked: Set to "true" for lock bucket and "false" for unlock
        bucket.
        :param namespace: The namespace
        """
        payload = {}

        if namespace:
            payload['namespace'] = namespace

        log.info("Setting lock to '{0}' for bucket '{1}': {2}".format(
                  is_locked, bucket_name, payload))

        return self.conn.put(
            url='object/bucket/{0}/lock/{1}'.format(bucket_name, is_locked),
            json_payload=payload
        )

    def update_bucket_quota(self, bucket_name, block_size, notification_size,
                            namespace=None):
        """
        Updates the quota for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: Name of the bucket for which the quota is to be
        updated
        :param block_size:
        :param notification_size:
        :param namespace: The namespace
        """
        payload = {
            "blockSize": block_size,
            "notificationSize": notification_size
        }

        if namespace:
            payload['namespace'] = namespace

        log.info("Updating quota for bucket '{0}': {1}".format(bucket_name,
                                                               payload))
        return self.conn.put(
            url='object/bucket/{0}/quota'.format(bucket_name),
            json_payload=payload
        )

    def get_bucket_quota(self, bucket_name, namespace=None):
        """
        Gets the quota for the given bucket and namespace. The namespace with
        which the bucket is associated can be specified as a query parameter.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'blockSize': 1,
            u'notificationSize': 2,
            u'namespace': u'namespace1',
            u'bucketname': u'bucket-test1'
        }

        :param bucket_name: Name of the bucket which for which quota is to be
        retrieved
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used.
        """
        msg = "Getting quota for bucket '{0}'".format(bucket_name)
        url = 'object/bucket/{0}/quota'.format(bucket_name)

        if namespace:
            url += '?namespace={0}'.format(namespace)
            msg += " in namespace '{0}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def delete_bucket_quota(self, bucket_name, namespace=None):
        """
        Deletes the quota setting for the given bucket and namespace.
        The namespace with which the bucket is associated can be specified as
        a query parameter.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: Name of the bucket for which the quota is to
        be deleted
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used.
        """
        params = None

        if namespace:
            params = {'namespace': namespace}

        log.info("Deleting quota for bucket '{0}'".format(bucket_name))

        return self.conn.delete(
            url='object/bucket/{0}/quota'.format(bucket_name),
            params=params
        )

    def get_bucket_acl(self, bucket_name, namespace=None):
        """
        Gets the ACL for the given bucket. Current user's namespace is used.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'namespace': u'namespace1',
            u'bucket': u'bucket-test1',
            u'permission': []
        }

        :param bucket_name: Name of the bucket for which ACL is to be updated.
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used.
        """
        params = None

        if namespace:
            params = {'namespace': namespace}

        log.info("Getting ACL for bucket '{0}'".format(bucket_name))

        return self.conn.get(
            url='object/bucket/{0}/acl'.format(bucket_name),
            params=params
        )

    def get_acl_permissions(self):
        """
        Gets all ACL permissions.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'permission': [
                {
                    u'display_name': u'read',
                    u'id': u'read'
                },
                {
                    u'display_name': u'readACL',
                    u'id': u'read_acl'
                },
                {
                    u'display_name': u'write',
                    u'id': u'write'
                },
                {
                    u'display_name': u'writeACL',
                    u'id': u'write_acl'
                },
                {
                    u'display_name': u'execute',
                    u'id': u'execute'
                },
                {
                    u'display_name': u'fullcontrol',
                    u'id': u'full_control'
                },
                {
                    u'display_name': u'privilegedwrite',
                    u'id': u'privileged_write'
                },
                {
                    u'display_name': u'delete',
                    u'id': u'delete'
                },
                {
                    u'display_name': u'none',
                    u'id': u'none'
                }
            ]
        }
        """
        log.info('Getting all ACLs')
        return self.conn.get(url='object/bucket/acl/permissions')

    def get_acl_groups(self):
        """
        Gets all ACL groups.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'group': [
                {
                    u'display_name': u'public',
                    u'id': u'public',
                    u'description': u'allusersincludingauthenticatedusers&anon'
                },
                {
                    u'display_name': u'allusers',
                    u'id': u'all_users',
                    u'description': u'allauthenticatedusers'
                },
                {
                    u'display_name': u'logdelivery',
                    u'id': u'log_delivery',
                    u'description': u'specifictoS3'
                },
                {
                    u'display_name': u'other',
                    u'id': u'other',
                    u'description': u'allauthenticatedusersbuttheowner'
                }
            ]
        }
        """
        log.info('Getting all ACL groups')
        return self.conn.get(url='object/bucket/acl/groups')



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class Certificate(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_certificate_chain(self):
        """
        Gets the certificate chain currently used. The data returned will
        contain all of the certificates in the current certificate chain
        encoded in PEM format.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            u'chain': u'-----BEGINCERTIFICATE-----\nMIIEnTCCA4WgAwI-
            REALLY-LONG-CERT\r\n-----ENDCERTIFICATE-----'
        }
        """
        log.info("Fetching certificate chain")
        return self.conn.get('object-cert/keystore')

    def put_certificate_chain(self):
        """
        Sets private key and certificate pair. The new certificate and key
        will be rotated into all of the nodes within 1 hour.
        """
        log.error("Certificate chain installation not supported")
        raise NotImplementedError



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class CallHome(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_connectemc_config(self):
        """
        Gets ConnectEMC configuration details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            "port": "25",
            "password": "",
            "transpport": "SMTP",
            "bsafe_encryption_ind": "no",
            "email_server": "mailhub.lss.emc.com",
            "email_sender": "someone@email.com",
            "notify_email_address": "johndoe@nowhere.com",
            "smtp_auth_type": "",
            "username": "",
            "start_tls_ind": "no",
            "enable_tls_cert": "no"
        }
        """
        log.info('Getting callhome (connectemc) config')
        return self.conn.get(url='vdc/callhome/connectemc/config')



"""

Buckets can be inserted into bucket_stats_info owned by DEFAULT_NO_OWNER
when ecr doesn't receive a response in time back from ecs.

In these cases, we need to:
- Identify the DEFAULT_NO_OWNER records in bucket_stats_info
- For each record, query ecs to find if the bucket does actually exist
- If it does, insert the owner record to object_user_bucket_owner
- Update bucket_stats_info and bucket_stats_hourly with the correct owner

Usage:
    reconcile.py [--conf <conf>] [--start <start> --end <end>]
    [--verbose]

Options:
    -h --help           Show this screen.
    --version           Script version.
    --verbose           Turn on verbose logging
    --start             When to reconcile from (int hours prior to now)
    --conf=<conf>       Location of the YAML config
                        [default: /etc/silenus/config.yaml]
"""

# Standard lib imports
import logging

# Third-party imports
import docopt
from tendo import singleton
import sqlalchemy
from psycopg2 import errorcodes as pg_errorcodes
from sqlalchemy import (
    exc as sa_exc,
)

# Project-level imports
from silenus import config
from silenus import (
    database_repo_factory as db_repos
)
from silenus.api import errors
from ecsminion import ECSMinion, ECSMinionException


LOG = logging.getLogger(__name__)


def load_config(config_path):
    """
    Load and parse YAML file.
    """
    config.register_defaults()
    config.load_config_from_file(config_path)

    return config.get_config()


def find_bucket_owner(app_config, bucket_name):
    """"
    Lookup the owner of an bucket in ECS if it exists

    Returns the string name of the owner or None if not found or
    an error occurs
    """
    if not bucket_name:
        return None

    try:
        app_settings = app_config['flask']['SILENUS']['vipr_api']
        namespace = app_config['flask']['SILENUS']['fixed_namespace']

        ecs_minion = ECSMinion(username=app_settings['username'],
                               password=app_settings['password'],
                               token_endpoint=app_settings['token_endpoint'],
                               ecs_endpoint=app_settings['ecs_endpoint'],
                               request_timeout=app_settings['request_timeout'])

        json_bucket_metadata = ecs_minion.bucket.get_bucket_info(
            bucket_name=bucket_name,
            namespace=namespace
        )

        if json_bucket_metadata:
            return json_bucket_metadata['owner']

    except ECSMinionException as ecsminion_ex:
        LOG.error('ECS API Message: {0} Status Code: {1}'.format(
            ecsminion_ex.ecs_message, ecsminion_ex.http_status_code)
        )
    except Exception as ex:
        LOG.error(ex.message)

    return None


def retrieve_info_buckets_with_missing_owners(db_repo, hours):
    """
    Query to find existing info buckets with DEFAULT_NO_OWNER
    """
    missing_buckets = db_repo['bucket_stats_info'].missing_owner_query(hours)
    return missing_buckets


def check_and_insert_object_user(db_repo, bucket_owner, bucket_name):
    # Check if we need to insert a new user
    object_user = retrieve_object_user(db_repo, bucket_owner)
    if not object_user:
        tenant_admin_id = retrieve_tenant_admin_id(db_repo,
                                                   bucket_owner, bucket_name)
        # Object user has never been created before
        if tenant_admin_id:
            object_user = db_repo['object_user'].insert_record(
                user_name=bucket_owner,
                tenant_admin=tenant_admin_id
            )
        else:
            LOG.warn('Unable to retrieve tenant admin in order to insert '
                     'bucket_owner:{0}, bucket_name: '
                     '{1}'.format(bucket_owner, bucket_name))

    return object_user


def insert_owner_association(db_repo, bucket_owner, bucket_name):
    object_user = check_and_insert_object_user(db_repo, bucket_owner,
                                               bucket_name)
    if not object_user:
        return

    inserted_record = None

    try:
        inserted_record = db_repo['bucket_owner'].insert_record(
            object_user_name=bucket_owner,
            bucket_name=bucket_name,
        )

    except sa_exc.IntegrityError as exc:
        _handle_integrity_exception(exc)

    if inserted_record:
        db_repo['object_user'].increment_bucket_count(
            inserted_record.object_user_name)

        LOG.debug('Inserted owner association:{0}, bucket_name:{1}'
                  'to object_user_bucket_owner'
                  .format(inserted_record.object_user_name,
                          inserted_record.bucket_name,
                          inserted_record.bucket_location,
                          inserted_record.bucket_location_owner))
    return inserted_record

def retrieve_object_user(db_repo, bucket_owner):
    object_user = db_repo['object_user'].retrieve_record(
        user_name=bucket_owner)
    return object_user


def retrieve_tenant_admin_id(db_repo, bucket_owner, bucket_name):
    model = db_repo['bucket_stats_info'].get_model()
    info_record = db_repo['bucket_stats_info'].retrieve_record(
        bucket_owner=bucket_owner, bucket_name=bucket_name)
    tenant_admin_id = None
    if info_record:
        tenant_admin_id = model.tenant_admin_id
    return tenant_admin_id


def _handle_integrity_exception(exc):
    # Handle duplicate value exceptions separately, so that a
    # 409 CONFLICT can be returned.
    if exc.orig.pgcode == pg_errorcodes.UNIQUE_VIOLATION:
        LOG.warn(
            'The bucket ownership relation already exists'
        )

    else:
        LOG.warn(
            'Error inserting association: {0}'.format(exc.message)
        )


def update_info_details(db_repo, bucket_name, bucket_owner):
    LOG.debug('Updating bucket_stats_info with bucket {0}, owner {1}'
              .format(bucket_name, bucket_owner))
    model = db_repo['bucket_stats_info'].get_model()
    try:
        db_repo['bucket_stats_info'].update_record(
            attributes={
                'bucket_owner': bucket_owner,
            },
            constraints=[
                model.bucket_name == bucket_name
            ]
        )
    # pylint: disable=W0703
    except Exception as ex:
        # No matter the error we want to log it
        LOG.error(
            'Error updating bucket_stats_info: {0}'.format(ex.message)
        )


def update_hourly_details(db_repo, bucket_name, bucket_owner):
    LOG.debug('Updating bucket_stats_hourly with bucket {0}, owner {1}'
              .format(bucket_name, bucket_owner))
    model = db_repo['bucket_stats_hourly'].get_model()
    try:
        db_repo['bucket_stats_hourly'].update_record(
            attributes={
                'bucket_owner': bucket_owner,
            },
            constraints=[
                model.bucket_name == bucket_name
            ]
        )
    # pylint: disable=W0703
    except Exception as ex:
        # No matter the error we want to log it
        LOG.error(
            'Error updating bucket_stats_hourly: {0}'.format(ex.message)
        )


def process(db_repo, app_config, hours):
    missing_buckets = ['testbucket-3dcfac67-dd4c-404f-82ab-235587417228'] #retrieve_info_buckets_with_missing_owners(db_repo, hours)
    if not missing_buckets:
        return
    LOG.info('Buckets no owners: {0}'.format(missing_buckets))

    buckets_to_be_added = {}
    for bucket_name in missing_buckets:
        LOG.debug('Querying ecs for bucket: ' + str(bucket_name))
        found_bucket_owner = find_bucket_owner(app_config, bucket_name)
        if found_bucket_owner:
            buckets_to_be_added[bucket_name] = found_bucket_owner

    LOG.info('These buckets and owners were found in ecs: ')
    for bucket_name, bucket_owner in buckets_to_be_added.items():
        LOG.info('bucket_name: {0} '
                 'bucket_owner: {1}'.format(bucket_name, bucket_owner))
    LOG.info('About to update silenus with the correct owners: ')

    for bucket_name, bucket_owner in buckets_to_be_added.items():
        """
        For records which exist in ecs, but have the wrong owner in silenus:
            1. Insert the owner record to object_user_bucket_owner
            2. Update bucket_stats_info with the correct owner
            3. Update bucket_stats_hourly with the correct owner
        """
        inserted = insert_owner_association(db_repo, bucket_owner, bucket_name)
        if inserted:
            update_info_details(db_repo, bucket_owner, bucket_name)
            update_hourly_details(db_repo, bucket_owner, bucket_name)


def main():
    # Exit if another version of this script is already running
    app_instance = singleton.SingleInstance()

    # Parse command line options and load config
    arguments = docopt.docopt(__doc__, version='Gouda Reconcile 1.0')
    app_config = load_config(arguments['--conf'])
    app_settings = app_config['reconcile']
    ecs_app_settings = app_config['flask']

    # Logging settings
    log_level = app_settings['logging']['log_level']
    log_file = app_settings['logging']['log_file']

    # If verbose is requested, switch log level to debug
    if arguments['--verbose']:
        log_level = logging.DEBUG

    config.configure_logger(LOG, log_level, log_file)

    # Create SQLAlchemy session
    engine = sqlalchemy.create_engine(
        app_config['flask']['SQLALCHEMY_DATABASE_URI'],
        pool_recycle=app_config['flask']['SQLALCHEMY_POOL_RECYCLE']
    )

    scoped_session = sqlalchemy.orm.scoped_session(
        sqlalchemy.orm.sessionmaker(bind=engine)
    )
    session = scoped_session()

    repo_factory = db_repos.DatabaseRepoFactory(session)
    db_repo = {
        'bucket_stats_info': repo_factory.get_repo('bucket_stats_info'),
        'bucket_stats_hourly': repo_factory.get_repo('bucket_stats_hourly'),
        'object_user': repo_factory.get_repo('object_user'),
        'bucket_owner': repo_factory.get_repo('bucket_owner')
    }

    LOG.info("Lock file located at: {0}".format(app_instance.lockfile))

    start = arguments['--start']
    if not start:
        # Default to 1 day in the past
        start = 24

    LOG.info("Starting reconciliation of buckets in ecs")
    process(db_repo, app_config, start)
    LOG.info("Reconciliation of buckets in ecs has completed")


if __name__ == "__main__":
    main()



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class SecretKeySelfService(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_user_secret_keys(self):
        """
        Gets all configured secret keys for the user account that makes the
        request.

        Required role(s):

        This call has no restrictions.

        Example JSON result from the API:

        {
          "user_secret_keys": {
            "secret_key_1": "",
            "key_timestamp_1": "",
            "key_expiry_timestamp_1": "",
            "secret_key_2": "",
            "key_timestamp_2": "",
            "key_expiry_timestamp_2": ""
          }
        }
        """
        log.info('Getting all secrets for current api user (me)')
        return self.conn.get(url='object/secret-keys')

    def create_new_secret_key(self, key_expiration=2592000):
        """
        Create a secret key for the authenticated user that makes the request.
        When creating new secret key, you may pass in an expiration time in
        minutes for the old key. During the expiration interval, both keys
        will be accepted for requests. This gives you a grace period where you
        can update applications to use the new key.

        Required role(s):

        This call has no restrictions.

        Example JSON result from the API:

        {
            "secret_key":"b2qffCUYCyyKrwoaEKkb1XoYB4m82banbgwUjjxs",
            "key_timestamp":"2015-09-30 20:57:59.149",
            "key_expiry_timestamp":"2015-09-30 20:57:59.149",
            "link":{
                    "rel":"self",
                    "href":"/object/user-secret-keys/root"
                    }
        }

        :param key_expiration: Expiry time/date for the secret
        key in minutes. Note that nodes may cache old keys for up to two
        minutes so the old key may not expire immediately. Defaults to 30 days
        (2592000 seconds)
        """
        payload = {
            "existing_key_expiry_time_mins": key_expiration

        }
        log.info('Creating secret for current api user (me)')
        return self.conn.post(url='object/secret-keys', json_payload=payload)

    def deactivate_user_secret_key(self, secret_key=None):
        """
        Deletes the specified secret key for the authenticated user that makes
        the request.

        Required role(s):

        This call has no restrictions.

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param secret_key: The secret key to deactivate (delete)
        """
        payload = {
            "secret_key": secret_key
        }

        log.info("Deleting secret for current api user (me)")

        return self.conn.post(
            url='object/secret-keys/deactivate', json_payload=payload)



# Standard lib imports
import httplib
import unittest

# Third party imports
from mock import MagicMock
from mock import patch

# Project level imports
from ecsminion import ECSMinion
from ecsminion.util.exceptions import ECSMinionException


def suite():
    test_suite = unittest.TestSuite()
    test_suite.addTest(WhenTestingNode())
    return test_suite


class WhenTestingNode(unittest.TestCase):

    def setUp(self):
        self.ecs_endpoint = 'https://127.0.0.1:4443'
        self.token_endpoint = 'https://127.0.0.1:4443/login'

        self.client = ECSMinion(
            ecs_endpoint=self.ecs_endpoint,
            token_endpoint=self.token_endpoint
        )

        self.returned_json = {
            "node": [
                {
                    "ip": "172.29.3.148",
                    "version": "1.2.0.0.60071.ffbe16c",
                    "rackId": "gray",
                    "nodename": "supr01-r01-01.lax01s1.rspaas-lab.ops.com",
                    "nodeid": "171.29.3.140"
                },
                {
                    "ip": "172.29.3.149",
                    "version": "1.2.0.0.60071.ffbe16c",
                    "rackId": "gray",
                    "nodename": "supr01-r01-02.lax01s1.rspaas-lab.ops.com",
                    "nodeid": "171.29.3.141"
                }
            ]
        }

        self.response = MagicMock()

    def test_get_nodes_should_throw_ecsminionexception(self):
        self.response.status_code = httplib.INTERNAL_SERVER_ERROR
        self.requests = MagicMock(return_value=self.response)
        self.requests.get.side_effect = [self.response]

        with patch('ecsminion.util.token_request.TokenRequest.get_new_token',
                   return_value='FAKE-TOKEN-123'):
            with patch('ecsminion.util.token_request.requests.Session.get'):
                with self.assertRaises(ECSMinionException):
                    self.client.node.get_nodes()

    def test_get_nodes(self):
        self.response.status_code = httplib.OK
        self.response.body = self.returned_json
        self.response.json = MagicMock(return_value=self.returned_json)
        self.requests = MagicMock(return_value=self.response)
        self.requests.get.side_effect = [self.response]

        with patch('ecsminion.util.token_request.TokenRequest.'
                   '_get_existing_token', return_value='FAKE-TOKEN-123'):
            with patch('ecsminion.requests.Session.get', self.requests):
                returned_json = self.client.node.get_nodes()
                self.assertEquals(returned_json, self.returned_json)

if __name__ == '__main__':
    unittest.main()



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class ManagementUser(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def create_local_user_info(self, uid, password, is_system_admin=False,
                               is_system_monitor=False):
        """
        Creates local users for the VDC. These users can be assigned to
        VDC-wide management roles and are not associated with a namespace.
        User account can be assigned to the System Admin role by setting the
        isSystemAdmin flag in the request payload.

        Required role(s):

        SYSTEM_ADMIN

        :param uid: Management user id, ex: sampleuser
        :param password: Password for the management user
        :param is_system_admin: If set to true, assigns the management user to
        the System Admin role
        :param is_system_monitor: If set to true, assigns the management user
        to the System Monitor role
        """
        payload = {
            "user": uid,
            "password": password,
            "is_system_admin": is_system_admin,
            "is_system_monitor": is_system_monitor
        }

        log.info("Creating local user '{0}': {1}".format(uid, payload))
        return self.conn.post(url='vdc/users', json_payload=payload)

    def delete_local_user_info(self, uid):
        """
        Deletes local management user information for the specified user
        identifier.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param uid: User identifier for which local user information needs to
        be deleted.
        """
        log.info("Deleting local user '{0}'".format(uid))
        return self.conn.post(url='vdc/users/{0}/deactivate'.format(uid))

    def modify_local_user_info(self, uid, password, is_system_admin=False,
                               is_system_monitor=False):
        """
        Updates user details for the specified local management user.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param uid: User identifier for which local user information needs to
        be updated.
        :param password: Password for the management user
        :param is_system_admin: Assigns or removes management user to /from
        System Admin role.
        :param is_system_monitor: Assigns or removes management user to /from
        System Monitor role.
        """
        payload = {
            "user": uid,
            "password": password,
            "is_system_admin": is_system_admin,
            "is_system_monitor": is_system_monitor
        }

        log.info("Updating local user '{0}': {1}".format(uid, payload))

        return self.conn.put(
            url='vdc/users/{0}'.format(uid), json_payload=payload)

    def get_local_user_info(self, uid):
        """
        Gets details for the specified local management user.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'isSystemMonitor': False,
            u'userId': u'admin',
            u'isSystemAdmin': True
        }

        :param uid: User identifier for which local user information needs to
        be retrieved
        """
        log.info("Getting local user '{0}'".format(uid))
        return self.conn.get(url='vdc/users/{0}'.format(uid))

    def get_local_management_users(self):
        """
        Gets all configured local management users.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'mgmt_user_info': [
                {
                    u'isSystemMonitor': False,
                    u'userId': u'someone@internal',
                    u'isSystemAdmin': True
                },
                {
                    u'isSystemMonitor': False,
                    u'userId': u'root',
                    u'isSystemAdmin': True
                }
            ]
        }
        """
        log.info('Getting all local management users')
        return self.conn.get(url='vdc/users')



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class VirtualDataCenter(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_all_vdcs(self):
        """
        Gets all details of all configured VDCs.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'vdc': [
                {
                    u'remote': None,
                    u'name': u'tiva01',
                    u'tags': [

                    ],
                    u'global': None,
                    u'interVdcEndPoints': u'192.29.3.48,
                    172.29.3.149,
                    172.29.3.150,
                    172.29.3.151,
                    172.29.3.152,
                    172.29.3.153,
                    172.29.3.154,
                    172.29.3.155',
                    u'vdcId': u'urn: storageos: VirtualDataCenterData:
                                a9faea85-d377-4a42-b5f1-fa15829f0c33',
                    u'vdc': None,
                    u'inactive': False,
                    u'link': {
                        u'href': u'/object/vdcs/vdc/tiva01',
                        u'rel': u'self'
                    },
                    u'secretKeys': u'55fmIFBniRuCBVx327Av',
                    u'vdcName': u'tiva01',
                    u'local': True,
                    u'id': u'urn: storageos: VirtualDataCenterData:
                                a9faea85-d377-4a42-b5f1-fa15829f0c33',
                    u'permanentlyFailed': False
                },
                {
                    u'remote': None,
                    u'name': u'tiva02',
                    u'tags': [

                    ],
                    u'global': None,
                    u'interVdcEndPoints': u'192.29.3.12,
                    172.29.3.213,
                    172.29.3.214,
                    172.29.3.215,
                    172.29.3.216,
                    172.29.3.217,
                    172.29.3.218,
                    172.29.3.219',
                    u'vdcId': u'urn: storageos: VirtualDataCenterData:
                                7d7d2ed5-e253-4be9-b8bb-5ff5b2697e6f',
                    u'vdc': None,
                    u'inactive': False,
                    u'link': {
                        u'href': u'/object/vdcs/vdc/tiva02',
                        u'rel': u'self'
                    },
                    u'secretKeys': u'99TilKh9aTwt9fLbCiKd',
                    u'vdcName': u'tiva02',
                    u'local': False,
                    u'id': u'urn: storageos: VirtualDataCenterData:
                                7d7d2ed5-e253-4be9-b8bb-5ff5b2697e6f',
                    u'permanentlyFailed': False
                }
            ]
        }
        """
        log.info('Getting all VDCs')
        return self.conn.get(url='object/vdcs/vdc/list')

    def get_vdc_by_id(self, vdc_id):
        """
        Gets the details for a VDC the identity of which is specified by
        its ID.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'tiva01',
            u'tags': [

            ],
            u'global': None,
            u'interVdcEndPoints': u'192.29.3.48,
            172.29.3.149,
            172.29.3.150,
            172.29.3.151,
            172.29.3.152,
            172.29.3.153,
            172.29.3.154,
            172.29.3.155',
            u'vdcId': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'vdc': None,
            u'inactive': False,
            u'link': {
                u'href': u'/object/vdcs/vdc/tiva01',
                u'rel': u'self'
            },
            u'secretKeys': u'55fmIFBniRuCBVx327Av',
            u'vdcName': u'tiva01',
            u'local': True,
            u'id': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'permanentlyFailed': False
        }

        param: vdc_id: VDC identifier for which VDC Information is to be
        retrieved.
        """
        log.info("Getting VDC by id '{0}'".format(vdc_id))
        return self.conn.get(url='object/vdcs/vdcid/{0}'.format(vdc_id))

    def get_local_vdc(self):
        """
        Gets the details for the local VDC.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'tiva01',
            u'tags': [

            ],
            u'global': None,
            u'interVdcEndPoints': u'192.29.3.48,
            172.29.3.149,
            172.29.3.150,
            172.29.3.151,
            172.29.3.152,
            172.29.3.153,
            172.29.3.154,
            172.29.3.155',
            u'vdcId': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'vdc': None,
            u'inactive': False,
            u'link': {
                u'href': u'/object/vdcs/vdc/tiva01',
                u'rel': u'self'
            },
            u'secretKeys': u'55fmIFBniRuCBVx327Av',
            u'vdcName': u'tiva01',
            u'local': True,
            u'id': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'permanentlyFailed': False
        }
        """
        log.info('Getting local VDC info')
        return self.conn.get(url='object/vdcs/vdc/local')

    def get_local_vdc_secret_key(self):
        """
        Gets the secret key for the local VDC.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {u'key': u'55fmIFBniRuCBVx327Av'}
        """
        log.info('Getting local VDC secret key')
        return self.conn.get(url='object/vdcs/vdc/local/secretkey')

    def get_vdc_by_name(self, vdc_name):
        """
        Gets the details for a VDC the identity of which is specified by
        its name.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'tiva01',
            u'tags': [

            ],
            u'global': None,
            u'interVdcEndPoints': u'192.29.3.48,
            172.29.3.149,
            172.29.3.150,
            172.29.3.151,
            172.29.3.152,
            172.29.3.153,
            172.29.3.154,
            172.29.3.155',
            u'vdcId': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'vdc': None,
            u'inactive': False,
            u'link': {
                u'href': u'/object/vdcs/vdc/tiva01',
                u'rel': u'self'
            },
            u'secretKeys': u'55fmIFBniRuCBVx327Av',
            u'vdcName': u'tiva01',
            u'local': True,
            u'id': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'permanentlyFailed': False
        }

        param: vdc_name: VDC name for which VDC Information is to be retrieved
        """
        log.info("Getting VDC by name '{0}'".format(vdc_name))
        return self.conn.get(url='object/vdcs/vdc/{0}'.format(vdc_name))

    def insert_vdc_attributes(self, vdc_name, inter_vdc_end_point,
                              secret_key):
        """
        Insert the attributes for the current VDC or a VDC which you want the
        current VDC to connect. Enables the name of the VDC, the end points
        that can be used to communicate with it, and a secret key used to
        encrypt traffic between VDCs to be set.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param vdc_name: Name of VDC to be inserted
        :param inter_vdc_end_point: End points for the VDC
        :param secret_key: Secret key to encrypt communication between VDC
        """
        payload = {
            "vdc_name": vdc_name,
            "inter_vdc_end_point": inter_vdc_end_point,
            "secret_key": secret_key
        }

        log.info("Updating VDC '{0}': {1}".format(vdc_name, payload))

        return self.conn.put(
            url='object/vdcs/vdc/{0}'.format(vdc_name), json_payload=payload)

    def deactivate_vdc(self, vdc_id):
        """
        Deactivates and deletes a VDC. Enables attributes for the current VDC
        to be deleted and enables information held by a VDC about other VDCs
        to be deleted.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param vdc_id: VDC identifier for which VDC Information needs to be
        deleted
        """
        log.info("Deleting VDC '{0}'".format(vdc_id))

        return self.conn.post(
            url='object/vdcs/vdc/{0}/deactivate'.format(vdc_id))



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class Namespace(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_namespaces(self):
        """
        Gets the identifiers for all configured namespaces.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'namespace': [
                {
                    u'link': {
                        u'href': u'/object/namespaces/namespace/namespace1',
                        u'rel': u'self'
                    },
                    u'name': u'namespace1',
                    u'id': u'namespace1'
                }
            ]
        }
        """
        log.info("Getting all namespaces")
        return self.conn.get(url='object/namespaces')

    def get_namespace(self, namespace):
        """
        Gets the details for the given namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'default_data_services_vpool':
                u'urn: storageos: ReplicationGroupInfo:
                    4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
            u'remote': None,
            u'name': u'namespace1',
            u'tags': [

            ],
            u'namespace_admins': u'someone@email.com',
            u'user_mapping': [

            ],
            u'global': None,
            u'disallowed_vpools_list': [

            ],
            u'vdc': None,
            u'inactive': False,
            u'link': {
                u'href': u'/object/namespaces/namespace/namespace1',
                u'rel': u'self'
            },
            u'allowed_vpools_list': [

            ],
            u'id': u'namespace1'
        }

        :param namespace: Namespace identifier for which details needs to
        be retrieved.
        """
        log.info("Getting info for namespace '{0}'".format(namespace))

        return self.conn.get(
            url='object/namespaces/namespace/{0}'.format(namespace))

    def get_retention_classes(self, namespace):
        """
        Gets the list of retention classes for the specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {u'retention_class': []}

        :param namespace: Namespace identifier for which retention classes
        needs to retrieved
        """
        log.info("Getting retention for namespace '{0}'".format(namespace))

        return self.conn.get(
            url='object/namespaces/namespace/{0}/retention'.format(namespace))

    def get_retention_class(self, name, namespace):
        """
        Gets the retention period for the given namespace and retention class.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            "name": "class_9cc8777c-bce0-11e4-8580-0050569c6fd7",
            "period": 2
        }

        :param name: Class name for which retention period needs to
        be retrieved
        :param namespace: Namespace for which retention period needs to
        be retrieved
        """
        log.info("Getting retention '{0}' in ns {1}".format(name, namespace))

        return self.conn.get(
            url='object/namespaces/namespace/{0}/retention/{1}'.format(
                namespace, name))

    def create_retention_class(self, name, period, namespace):
        """
        Creates a retention class for the specified namespace. The method
        payload specifies the retention class details which define a name for
        the class and a retention period.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param name: Name of the retention class
        :param period: Period of the retention class in seconds
        :param namespace: Namespace identifier for which retention class needs
        to be created.
        """
        payload = {
            "name": name,
            "period": period
        }

        log.info("Creating retention '{0}' in ns {1}".format(name, namespace))

        return self.conn.post(
            url='object/namespaces/namespace/{0}/retention'.format(namespace),
            json_payload=payload
        )

    def update_retention_class(self, name, period, namespace):
        """
        Updates the retention class details for a specified retention class
        for a namespace.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param name: Retention class name for which details needs to updated.
        :param period: A new period value for class in seconds
        :param namespace: Namespace identifier for which retention class needs
        to be retrieved.
        """
        payload = {
            "period": period
        }

        log.info("Updating retention '{0}' in ns {1}: {2}".format(name,
                                                                  namespace,
                                                                  payload))
        return self.conn.put(
            url='object/namespaces/namespace/{0}/retention/{1}'.format(
                namespace, name), json_payload=payload)

    def get_namespace_quota(self, namespace):
        """
        Gets the namespace quota for a specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'blockSize': -1,
            u'notificationSize': -1,
            u'namespace': u'namespace1'
        }

        :param namespace: Namespace identifier for which namespace quota
        details needs to retrieved.
        """
        log.info("Getting quota for namespace '{0}'".format(namespace))

        return self.conn.get(
            url='object/namespaces/namespace/{0}/quota'.format(namespace))

    def update_namespace_quota(self, block_size, notification_size, namespace):
        """
        Updates the namespace quota for a specified namespace.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param block_size: Block size in GB.
        :param notification_size: Notification size in GB.
        :param namespace: Namespace identifier for which namespace quota
        details need to be updated.
        """
        # ----
        # Note
        # ----
        # These two params are what the ECS API docs state, however, every
        # other parameter is broken up into two word with an underscore
        # separating them, ie: block_size
        # I can't verify at the moment that this call works as is.
        payload = {
            "blockSize": block_size,
            "notificationSize": notification_size
        }

        log.info("Updating quota for namespace '{0}': {1}".format(namespace,
                                                                  payload))
        return self.conn.put(
            url='object/namespaces/namespace/{0}/quota'.format(namespace),
            json_payload=payload
        )




"""-*- coding: utf-8 -*-
ECSMinion Setup
"""

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


def read(relative):
    """
    Read file contents and return a list of lines.
    ie, read the VERSION file
    """
    contents = open(relative, 'r').read()
    return [l for l in contents.split('\n') if l != '']

with open('README.rst', 'r') as f:
    readme = f.read()

setup(
    name='ecsminion',
    url='https://github.com/chadlung/ecsminion',
    keywords=['ecsminion'],
    long_description=readme,
    version=read('VERSION')[0],
    description='A library for interacting with the ECS 2.x Management API',
    author='Chad Lung',
    author_email='chad.lung@gmail.com',
    tests_require=read('./test-requirements.txt'),
    install_requires=read('./requirements.txt'),
    test_suite='nose.collector',
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=['ez_setup'])
)



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class DataStore(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_data_store_list(self):
        """
        Gets list of configured commodity or filesystem data stores.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'data_store': [
                {
                    u'link': {
                        u'href': u'/vdc/data-stores/commodity/192.29.3.51',
                        u'rel': u'self'
                    },
                    u'name': u'tiva01-r01-04.blah01s1.rraas-ops.com',
                    u'resource_type': u'commodity',
                    u'id': u'192.29.3.51'
                },
                {
                    u'link': {
                        u'href': u'/vdc/data-stores/commodity/192.29.3.54',
                        u'rel': u'self'
                    },
                    u'name': u'tiva01-r01-07.blah02s2.rraas-ops.com',
                    u'resource_type': u'commodity',
                    u'id': u'192.29.3.54'
                }
            ]
        }
        """
        log.info("Getting all data stores")
        return self.conn.get(url='vdc/data-stores')

    def get_commodity_data_store_associated_wth_storage_pool(
            self, commodity_node_id):
        """
        Gets the details for a commodity data store.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'tiva01-r01-04.blah01s1.rraas-ops.com',
            u'tags': [

            ],
            u'device_state': u'readytouse',
            u'global': None,
            u'creation_time': 1432185210000,
            u'device_info': u'',
            u'used_gb': 1073,
            u'usable_gb': 334800,
            u'vdc': None,
            u'link': {
                u'href': u'/vdc/object-pools/172.29.3.151',
                u'rel': u'self'
            },
            u'free_gb': 333727,
            u'varray': u'urn: storageos: VirtualArray:
                        3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
            u'id': u'192.29.3.51',
            u'description': u''
        }

        :param commodity_node_id: Identifier of the data store
        """
        log.info("Getting commodity store '{0}'".format(commodity_node_id))

        return self.conn.get(
            url='vdc/data-stores/commodity/{0}'.format(commodity_node_id))

    def get_commodity_data_store_associated_wth_varray(
            self, storage_pool_id):
        """
        Gets the list of details of commodity data stores associated with a
        storage pool.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'commodity_data_store': [
                {
                    u'remote': None,
                    u'name': u'tiva01-r01-04.blah01s1.rraas-ops.com',
                    u'tags': [

                    ],
                    u'device_state': u'readytouse',
                    u'global': None,
                    u'creation_time': 1432185210000,
                    u'device_info': u'',
                    u'used_gb': 318025,
                    u'usable_gb': 334800,
                    u'vdc': None,
                    u'link': {
                        u'href': u'/vdc/object-pools/192.29.3.51',
                        u'rel': u'self'
                    },
                    u'free_gb': 16775,
                    u'varray': u'urn: storageos: VirtualArray:
                                3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
                    u'id': u'192.29.3.51',
                    u'description': u''
                },
                {
                    u'remote': None,
                    u'name': u'tiva01-r01-07.blah02s2.rraas-ops.com',
                    u'tags': [

                    ],
                    u'device_state': u'readytouse',
                    u'global': None,
                    u'creation_time': 1432185214000,
                    u'device_info': u'',
                    u'used_gb': 318026,
                    u'usable_gb': 334800,
                    u'vdc': None,
                    u'link': {
                        u'href': u'/vdc/object-pools/192.29.3.54',
                        u'rel': u'self'
                    },
                    u'free_gb': 16774,
                    u'varray': u'urn: storageos: VirtualArray:
                                3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
                    u'id': u'192.29.3.54',
                    u'description': u''
                }
            ]
        }

        :param storage_pool_id: Identifier of the storage pool
        """
        log.info("Getting commodity store for varray '{0}'"
                 .format(storage_pool_id))
        return self.conn.get(
            url='vdc/data-stores/commodity/search/varray/{0}'.format(
                storage_pool_id))



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class StoragePool(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_virtual_array(self, storage_pool_id):
        """
        Gets the details for the specified storage pool.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'isProtected': False,
            u'id': u'urn: storageos: VirtualArray:
                    3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
            u'name': u'storagepool1'
        }

        :param storage_pool_id: Storage pool identifier to be retrieved
        """
        log.info("Getting varray '{0}'".format(storage_pool_id))

        return self.conn.get(
            url='vdc/data-services/varrays/{0}'.format(storage_pool_id))

    def get_virtual_arrays(self, vdc_id):
        """
        Gets a list of storage pools from the local VDC.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'varray': [
                {
                    u'isProtected': False,
                    u'id': u'urn: storageos: VirtualArray:
                            3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
                    u'name': u'storagepool1'
                }
            ]
        }

        :param vdc_id: Virtual data center identifier for which list of
        storage pool is to be retrieved
        """
        log.info("Getting varrays for VDC '{0}'".format(vdc_id))

        params = {
            'vdc-id': vdc_id
        }

        return self.conn.get(url='vdc/data-services/varrays', params=params)



# Standard lib imports
import datetime

# Third party imports
# None

# Project level imports
# None


def get_formatted_time_string(year, month, day, hour, minute=None):
    """
    Validates input parameters: year, month, day, hour and minute
    and returns the timestamp if valid

    :param year: Four digit year
    :param month: The month
    :param day: The day
    :param hour: The hour
    :param minute: The minutes (optional)
    :return: Returns time stamp in yyyy-MM-dd'T'HH:mm if parameters are valid

    Throws:
        ValueError in case of invalid input
    """

    if minute:
        d = datetime.datetime(int(year), int(month), int(day), int(hour),
                              int(minute))
        return d.strftime("%Y-%m-%dT%H:%M")
    else:
        d = datetime.datetime(int(year), int(month), int(day), int(hour))
        return d.strftime("%Y-%m-%dT%H")



# Standard lib imports
import httplib
import unittest

# Third party imports
from mock import MagicMock
from mock import mock_open
from mock import patch

# Project level imports
from ecsminion.util.token_request import TokenRequest
from ecsminion.util.exceptions import ECSMinionException


def suite():
    test_suite = unittest.TestSuite()
    test_suite.addTest(WhenTestingTokenRequest())
    return test_suite


class WhenTestingTokenRequest(unittest.TestCase):

    def setUp(self):
        self.token_file_contents = '123TOKEN'
        self.response = MagicMock()
        self.session_get = 'ecsminion.util.token_request.requests.Session.get'

        self.token_request = TokenRequest(username='someone',
                                          password='password',
                                          ecs_endpoint='https://localhost',
                                          token_endpoint='https://localhost',
                                          verify_ssl=False,
                                          token_filename='ecstoken.tkn',
                                          token_location='/tmp',
                                          request_timeout=5.0,
                                          cache_token=True)

    def test_should_get_existing_token(self):
        with patch('os.path.isfile', return_value=True),\
            patch('__builtin__.open', mock_open(read_data='123TOKEN'),
                  create=True):
            self.assertEqual(self.token_request._get_existing_token(),
                             self.token_file_contents)

    def test_should_not_get_existing_token(self):
        with patch('os.path.isfile', return_value=False):
            self.assertEqual(self.token_request._get_existing_token(), None)

    def test_get_new_token_should_throw_ecsminionexception_500(self):
        self.response.status_code = httplib.INTERNAL_SERVER_ERROR
        self.requests = MagicMock(return_value=self.response)
        self.requests.get.side_effect = [self.response]

        with patch(self.session_get, self.requests):
            with self.assertRaises(ECSMinionException):
                self.token_request.get_new_token()

    def test_get_new_token_should_throw_ecsminionexception_401(self):
        self.response.status_code = httplib.UNAUTHORIZED
        self.requests = MagicMock(return_value=self.response)
        self.requests.get.side_effect = [self.response]

        with patch(self.session_get, self.requests):
            with self.assertRaises(ECSMinionException):
                self.token_request.get_new_token()

if __name__ == '__main__':
    unittest.main()
